Type: Package
Package: magrittr
Title: A Forward-Pipe Operator for R
Version: 2.0.5
Authors@R: c(
    person("Stefan Milton", "Bache", , "stefan@stefanbache.dk", role = c("aut", "cph"),
           comment = "Original author and creator of magrittr"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Lionel", "Henry", , "lionel@posit.co", role = "cre"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
            comment = c(ROR = "03wc8by49"))
  )
Description: Provides a mechanism for chaining commands with a new
    forward-pipe operator, %>%. This operator will forward a value, or the
    result of an expression, into the next function call/expression.
    There is flexible support for the type of right-hand side expressions.
    For more information, see package vignette.  To quote Rene Magritte,
    "Ceci n'est pas un pipe."
License: MIT + file LICENSE
URL: https://magrittr.tidyverse.org,
        https://github.com/tidyverse/magrittr
BugReports: https://github.com/tidyverse/magrittr/issues
Depends: R (>= 3.4.0)
Suggests: covr, knitr, rlang, rmarkdown, testthat
VignetteBuilder: knitr
ByteCompile: Yes
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-03 08:09:48 UTC; lionel
Author: Stefan Milton Bache [aut, cph] (Original author and creator of
    magrittr),
  Hadley Wickham [aut],
  Lionel Henry [cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2026-04-04 08:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-08 00:57:51 UTC; unix
Archs: magrittr.so.dSYM

Type: Package
Package: magrittr
Title: A Forward-Pipe Operator for R
Version: 2.0.5
Authors@R: c(
    person("Stefan Milton", "Bache", , "stefan@stefanbache.dk", role = c("aut", "cph"),
           comment = "Original author and creator of magrittr"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Lionel", "Henry", , "lionel@posit.co", role = "cre"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
            comment = c(ROR = "03wc8by49"))
  )
Description: Provides a mechanism for chaining commands with a new
    forward-pipe operator, %>%. This operator will forward a value, or the
    result of an expression, into the next function call/expression.
    There is flexible support for the type of right-hand side expressions.
    For more information, see package vignette.  To quote Rene Magritte,
    "Ceci n'est pas un pipe."
License: MIT + file LICENSE
URL: https://magrittr.tidyverse.org,
        https://github.com/tidyverse/magrittr
BugReports: https://github.com/tidyverse/magrittr/issues
Depends: R (>= 3.4.0)
Suggests: covr, knitr, rlang, rmarkdown, testthat
VignetteBuilder: knitr
ByteCompile: Yes
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-03 08:09:48 UTC; lionel
Author: Stefan Milton Bache [aut, cph] (Original author and creator of
    magrittr),
  Hadley Wickham [aut],
  Lionel Henry [cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2026-04-04 08:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-04 10:13:02 UTC; unix
Archs: magrittr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("[",fseq)
S3method("[[",fseq)
S3method(print,fseq)
export("%!>%")
export("%$%")
export("%<>%")
export("%>%")
export("%T>%")
export("n'est pas")
export(add)
export(and)
export(debug_fseq)
export(debug_pipe)
export(divide_by)
export(divide_by_int)
export(equals)
export(extract)
export(extract2)
export(freduce)
export(functions)
export(inset)
export(inset2)
export(is_greater_than)
export(is_in)
export(is_less_than)
export(is_weakly_greater_than)
export(is_weakly_less_than)
export(mod)
export(multiply_by)
export(multiply_by_matrix)
export(not)
export(or)
export(pipe_eager_lexical)
export(pipe_lazy_masking)
export(pipe_nested)
export(raise_to_power)
export(set_attr)
export(set_attributes)
export(set_class)
export(set_colnames)
export(set_names)
export(set_rownames)
export(subtract)
export(undebug_fseq)
export(use_series)
useDynLib(magrittr, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# magrittr <a href='https://magrittr.tidyverse.org'><img src='man/figures/logo.png' align="right" height="139" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/magrittr)](https://cran.r-project.org/package=magrittr)
[![Codecov test
coverage](https://codecov.io/gh/tidyverse/magrittr/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/magrittr?branch=main)
[![R-CMD-check](https://github.com/tidyverse/magrittr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/tidyverse/magrittr/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## Overview

The magrittr package offers a set of operators which make your code more
readable by:

-   structuring sequences of data operations left-to-right (as opposed
    to from the inside and out),
-   avoiding nested function calls,
-   minimizing the need for local variables and function definitions,
    and
-   making it easy to add steps anywhere in the sequence of operations.

The operators pipe their left-hand side values forward into expressions
that appear on the right-hand side, i.e. one can replace `f(x)` with
`x %>% f()`, where `%>%` is the (main) pipe-operator. When coupling
several function calls with the pipe-operator, the benefit will become
more apparent. Consider this pseudo example:

``` r
the_data <-
  read.csv('/path/to/data/file.csv') %>%
  subset(variable_a > x) %>%
  transform(variable_c = variable_a/variable_b) %>%
  head(100)
```

Four operations are performed to arrive at the desired data set, and
they are written in a natural order: the same as the order of execution.
Also, no temporary variables are needed. If yet another operation is
required, it is straightforward to add to the sequence of operations
wherever it may be needed.

If you are new to magrittr, the best place to start is the [pipes
chapter](https://r4ds.had.co.nz/pipes.html) in R for data science.

## Installation

``` r
# The easiest way to get magrittr is to install the whole tidyverse:
install.packages("tidyverse")

# Alternatively, install just magrittr:
install.packages("magrittr")

# Or the development version from GitHub:
# install.packages("devtools")
pak::pak("tidyverse/magrittr")
```

## Usage

### Basic piping

-   `x %>% f` is equivalent to `f(x)`
-   `x %>% f(y)` is equivalent to `f(x, y)`
-   `x %>% f %>% g %>% h` is equivalent to `h(g(f(x)))`

Here, “equivalent” is not technically exact: evaluation is non-standard,
and the left-hand side is evaluated before passed on to the right-hand
side expression. However, in most cases this has no practical
implication.

### The argument placeholder

-   `x %>% f(y, .)` is equivalent to `f(y, x)`
-   `x %>% f(y, z = .)` is equivalent to `f(y, z = x)`

### Re-using the placeholder for attributes

It is straightforward to use the placeholder several times in a
right-hand side expression. However, when the placeholder only appears
in a nested expressions magrittr will still apply the first-argument
rule. The reason is that in most cases this results more clean code.

`x %>% f(y = nrow(.), z = ncol(.))` is equivalent to
`f(x, y = nrow(x), z = ncol(x))`

The behavior can be overruled by enclosing the right-hand side in
braces:

`x %>% {f(y = nrow(.), z = ncol(.))}` is equivalent to
`f(y = nrow(x), z = ncol(x))`

### Building (unary) functions

Any pipeline starting with the `.` will return a function which can
later be used to apply the pipeline to values. Building functions in
magrittr is therefore similar to building other values.

``` r
f <- . %>% cos %>% sin 
# is equivalent to 
f <- function(.) sin(cos(.)) 
```

### Pipe with exposition of variables

Many functions accept a data argument, e.g. `lm` and `aggregate`, which
is very useful in a pipeline where data is first processed and then
passed into such a function. There are also functions that do not have a
data argument, for which it is useful to expose the variables in the
data. This is done with the `%$%` operator:

``` r
iris %>%
  subset(Sepal.Length > mean(Sepal.Length)) %$%
  cor(Sepal.Length, Sepal.Width)
#> [1] 0.3361992

data.frame(z = rnorm(100)) %$%
  ts.plot(z)
```

![](man/figures/exposition-1.png)<!-- -->

## Code of Conduct

Please note that the magrittr project is released with a [Contributor
Code of Conduct](https://magrittr.tidyverse.org/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' Extract the function list from a functional sequence.
#'
#' This can be used to extract the list of functions inside a functional
#' sequence created with a chain like \code{. \%>\% foo \%>\% bar}.
#'
#' @param fseq A functional sequence ala magrittr.
#' @return a list of functions
#' 
#' @export
functions <- function(fseq)
{
  if (!"fseq" %in% class(fseq))
    stop("Object is not a functional sequence.", call. = FALSE)
  environment(fseq)[["_function_list"]]
}

#' Print method for functional sequence.
#'
#' @param x A functional sequence object
#' @param ... not used.
#' @return x
#'
#' @export
print.fseq <- function(x, ...)
{
  flist <- functions(x)

  cat("Functional sequence with the following components:\n\n")
  lapply(seq_along(flist), 
      function(i) cat(" ", i, ". ", deparse(body(flist[[i]])), "\n", sep = ""))
  cat("\nUse 'functions' to extract the individual functions.", "\n")
  x
}

#' magrittr - Ceci n'est pas un pipe
#'
#' The magrittr package offers a set of operators which promote semantics 
#' that will improve your code by
#' \itemize{
#'   \item structuring sequences of data operations left-to-right
#'         (as opposed to from the inside and out),
#'   \item avoiding nested function calls, 
#'   \item minimizing the need for local variables and function definitions, and
#'   \item making it easy to add steps anywhere in the sequence of operations.
#' }
#' The operators pipe their left-hand side values forward into expressions that
#' appear on the right-hand side, i.e. one can replace `f(x)` with 
#' \code{x \%>\% f}, where \code{\%>\%} is the (main) pipe-operator.
#' 
#' Consider the example below. Four operations are performed to 
#' arrive at the desired data set, and they are written in a natural order: 
#' the same as the order of execution. Also, no temporary variables are needed.
#' If yet another operation is required, it is straight-forward to add to the
#' sequence of operations whereever it may be needed.
#' 
#' For a more detailed introduction see the vignette 
#' (`vignette("magrittr")`) or the documentation pages for the
#' available operators:\cr
#' \tabular{ll}{
#'    \code{\link{\%>\%}}  \tab pipe.\cr
#'    \code{\link{\%T>\%}} \tab tee pipe.\cr
#'    \code{\link{\%<>\%}} \tab assignment pipe.\cr
#'    \code{\link{\%$\%}}  \tab exposition pipe.\cr
#' }
#' 
#' @useDynLib magrittr, .registration = TRUE
#' @examples
#' \dontrun{
#' 
#' the_data <-
#'   read.csv('/path/to/data/file.csv') %>%
#'   subset(variable_a > x) %>%
#'   transform(variable_c = variable_a/variable_b) %>%
#'   head(100)
#' }
#' @keywords internal
"_PACKAGE"

.onLoad <- function(lib, pkg) {
  .Call(magrittr_init, asNamespace("magrittr"))
}

#' Apply a list of functions sequentially
#'
#' This function applies the first function to `value`, then the
#' next function to the result of the previous function call, etc. 
#' 
#' @param value initial value.
#' @param function_list a list of functions.
#' @return The result after applying each function in turn.
#'
#'
#' @export
freduce <- function(value, function_list) {
  k <- length(function_list)
  if (k > 1) {
    for (i in 1:(k - 1L)) {
      value <- function_list[[i]](value)
    }
  }

  value <- withVisible(function_list[[k]](value))

  if (value[["visible"]]) {
    value[["value"]]
  } else {
    invisible(value[["value"]])
  }
}

#' Pipe
#' 
#' Pipe an object forward into a function or call expression.
#' 
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' @details
#' \subsection{Using `%>%` with unary function calls}{
#' 
#' When functions require only one argument, `x %>% f` is equivalent
#' to `f(x)` (not exactly equivalent; see technical note below.)
#' 
#' }
#' \subsection{Placing `lhs` as the first argument in `rhs` call}{
#' 
#' The default behavior of `%>%` when multiple arguments are required
#' in the `rhs` call, is to place `lhs` as the first argument, i.e. 
#' `x %>% f(y)` is equivalent to `f(x, y)`.
#' }
#' \subsection{Placing `lhs` elsewhere in `rhs` call}{
#' 
#' Often you will want `lhs` to the `rhs` call at another position than the first.
#' For this purpose you can use the dot (`.`) as placeholder. For example,
#' `y %>% f(x, .)` is equivalent to `f(x, y)` and
#' `z %>% f(x, y, arg = .)` is equivalent to `f(x, y, arg = z)`.
#' }
#' 
#' \subsection{Using the dot for secondary purposes}{
#' 
#' Often, some attribute or property of `lhs` is desired in the `rhs` call in
#' addition to the value of `lhs` itself, e.g. the number of rows or columns.
#' It is perfectly valid to use the dot placeholder several times in the `rhs`
#' call, but by design the behavior is slightly different when using it inside
#' nested function calls. In particular, if the placeholder is only used 
#' in a nested function call, `lhs` will also be placed as the first argument!
#' The reason for this is that in most use-cases this produces the most readable 
#' code. For example, `iris %>% subset(1:nrow(.) %% 2 == 0)` is 
#' equivalent to `iris %>% subset(., 1:nrow(.) %% 2 == 0)` but
#' slightly more compact. It is possible to overrule this behavior by enclosing
#' the `rhs` in braces. For example, `1:10 %>% {c(min(.), max(.))}` is
#' equivalent to `c(min(1:10), max(1:10))`.
#' }
#' 
#' \subsection{Using `%>%` with call- or function-producing `rhs`}{
#' 
#' It is possible to force evaluation of `rhs` before the piping of `lhs` takes 
#' place. This is useful when `rhs` produces the relevant call or function.
#' To evaluate `rhs` first, enclose it in parentheses, i.e. 
#' `a %>% (function(x) x^2)`, and `1:10 %>% (call("sum"))`.
#' Another example where this is relevant is for reference class methods
#' which are accessed using the `$` operator, where one would do
#' `x %>% (rc$f)`, and not `x %>% rc$f`.
#' }
#' 
#' \subsection{Using lambda expressions with `%>%`}{
#' 
#' Each `rhs` is essentially a one-expression body of a unary function.
#' Therefore defining lambdas in magrittr is very natural, and as 
#' the definitions of regular functions: if more than a single expression
#' is needed one encloses the body in a pair of braces, \code{\{ rhs \}}.
#' However, note that within braces there are no "first-argument rule":
#' it will be exactly like writing a unary function where the argument name is
#' "`.`" (the dot).
#' 
#' }
#' \subsection{Using the dot-place holder as `lhs`}{
#' 
#' When the dot is used as `lhs`, the result will be a functional sequence, 
#' i.e. a function which applies the entire chain of right-hand sides in turn 
#' to its input. See the examples.
#' }
#' 
#' @section Technical notes:
#' The magrittr pipe operators use non-standard evaluation. They capture
#' their inputs and examine them to figure out how to proceed. First a function
#' is produced from all of the individual right-hand side expressions, and 
#' then the result is obtained by applying this function to the left-hand side.
#' For most purposes, one can disregard the subtle aspects of magrittr's 
#' evaluation, but some functions may capture their calling environment, 
#' and thus using the operators will not be exactly equivalent to the 
#' "standard call" without pipe-operators.
#' 
#' 
#' Another note is that special attention is advised when using non-magrittr
#' operators in a pipe-chain (`+, -, $,` etc.), as operator precedence will impact how the 
#' chain is evaluated. In general it is advised to use the aliases provided 
#' by magrittr.
#' 
#' @seealso \code{\link{\%<>\%}}, \code{\link{\%T>\%}}, \code{\link{\%$\%}}
#' 
#' @examples
#' # Basic use:
#' iris %>% head
#' 
#' # Use with lhs as first argument
#' iris %>% head(10)
#' 
#' # Using the dot place-holder
#' "Ceci n'est pas une pipe" %>% gsub("une", "un", .)
#'   
#' # When dot is nested, lhs is still placed first:
#' sample(1:10) %>% paste0(LETTERS[.])
#' 
#' # This can be avoided:
#' rnorm(100) %>% {c(min(.), mean(.), max(.))} %>% floor
#' 
#' # Lambda expressions: 
#' iris %>%
#' {
#'   size <- sample(1:10, size = 1)
#'   rbind(head(., size), tail(., size))
#' }
#' 
#' # renaming in lambdas:
#' iris %>%
#' {
#'   my_data <- .
#'   size <- sample(1:10, size = 1)
#'   rbind(head(my_data, size), tail(my_data, size))
#' }
#' 
#' # Building unary functions with %>%
#' trig_fest <- . %>% tan %>% cos %>% sin
#' 
#' 1:10 %>% trig_fest
#' trig_fest(1:10)
#' 
#' @rdname pipe
#' @export
`%>%` <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 1L
  env <- parent.frame()
  lazy <- TRUE
  .External2(magrittr_pipe)
}

#' Lazy and eager pipes
#'
#' Assign these pipe variants to an infix symbol like `%>%`.
#'
#' @inheritParams %>%
#' @keywords internal
#' @export
pipe_eager_lexical <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 1L
  env <- parent.frame()
  sym <- sys.call()[[1]]
  .External2(magrittr_pipe)
}
#' @rdname pipe_eager_lexical
#' @export
pipe_lazy_masking <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 1L
  env <- parent.frame()
  lazy <- TRUE
  sym <- sys.call()[[1]]
  .External2(magrittr_pipe)
}
#' @rdname pipe_eager_lexical
#' @export
pipe_nested <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 1L
  env <- parent.frame()
  nested <- TRUE
  sym <- sys.call()[[1]]
  .External2(magrittr_pipe)
}

#' Assignment pipe
#' 
#' Pipe an object forward into a function or call expression and update the 
#' `lhs` object with the resulting value.
#' 
#' @param lhs An object which serves both as the initial value and as target.
#' @param rhs a function call using the magrittr semantics.
#' 
#' @details The assignment pipe, \code{\%<>\%}, is used to
#' update a value by first piping it into one or more `rhs` expressions, and 
#' then assigning the result. For example, \code{some_object \%<>\% 
#' foo \%>\% bar} is equivalent to \code{some_object <- some_object \%>\% foo
#' \%>\% bar}. It must be the first pipe-operator in a chain, but otherwise it
#' works like \code{\link{\%>\%}}.
#' 
#' @seealso \code{\link{\%>\%}}, \code{\link{\%T>\%}}, \code{\link{\%$\%}}
#' 
#' @examples
#' iris$Sepal.Length %<>% sqrt
#' 
#' x <- rnorm(100)
#' 
#' x %<>% abs %>% sort
#' 
#' is_weekend <- function(day)
#' {
#'    # day could be e.g. character a valid representation
#'    day %<>% as.Date
#'    
#'    result <- day %>% format("%u") %>% as.numeric %>% is_greater_than(5)
#'    
#'    if (result)
#'      message(day %>% paste("is a weekend!"))
#'    else
#'      message(day %>% paste("is not a weekend!"))
#'    
#'    invisible(result)
#' }
#' 
#' @rdname compound
#' @export
`%<>%` <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 2L
  lazy <- TRUE
  env <- parent.frame()
  .External2(magrittr_pipe)
}

#' Tee pipe
#' 
#' Pipe a value forward into a function- or call expression and return the
#' original value instead of the result. This is useful when an expression
#' is used for its side-effect, say plotting or printing.
#' 
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' 
#' @details The tee pipe works like \code{\link{\%>\%}}, except the 
#' return value is `lhs` itself, and not the result of `rhs` function/expression.
#' 
#' @seealso \code{\link{\%>\%}}, \code{\link{\%<>\%}}, \code{\link{\%$\%}}
#' 
#' @examples
#' rnorm(200) %>%
#' matrix(ncol = 2) %T>%
#' plot %>% # plot usually does not return anything. 
#' colSums
#' 
#' @rdname tee
#' @export
`%T>%` <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 3L
  lazy <- TRUE
  env <- parent.frame()
  .External2(magrittr_pipe)
}

#' Exposition pipe
#' 
#' Expose the names in `lhs` to the `rhs` expression. This is useful when functions
#' do not have a built-in data argument.
#' 
#' @param lhs A list, environment, or a data.frame.
#' @param rhs An expression where the names in lhs is available.
#' 
#' @details Some functions, e.g. `lm` and `aggregate`, have a 
#' data argument, which allows the direct use of names inside the data as part 
#' of the call. This operator exposes the contents of the left-hand side object
#' to the expression on the right to give a similar benefit, see the examples.

#' @seealso \code{\link{\%>\%}}, \code{\link{\%<>\%}}, \code{\link{\%T>\%}}
#' 
#' @examples
#' iris %>%
#'   subset(Sepal.Length > mean(Sepal.Length)) %$%
#'   cor(Sepal.Length, Sepal.Width)
#'   
#' data.frame(z = rnorm(100)) %$% 
#'   ts.plot(z)
#'   
#' @rdname exposition
#' @export
`%$%` <- function(lhs, rhs) {
  lhs <- substitute(lhs)
  rhs <- substitute(rhs)
  kind <- 4L
  lazy <- TRUE
  env <- parent.frame()
  .External2(magrittr_pipe)
}


new_lambda <- function(exprs, env) {
  `_function_list` <- lapply(exprs, as_pipe_fn, env)

  structure(
    function(value) freduce(value, `_function_list`),
    class = c("fseq", "function")
  )
}

lambda_fmls <- as.pairlist(alist(. = ))
as_pipe_fn <- function(expr, env) {
  eval(call("function", lambda_fmls, expr), env)
}

#' FAQ: What is the gender of the pipe?
#'
#' In Magritte's original quote "Ceci n'est pas une pipe," the word
#' "pipe" is feminine. However the magrittr package quotes it as
#' "Ceci n'est pas un pipe," with a masculine "pipe." This
#' lighthearted misappropriation is intentional. Whereas the
#' object represented in Magritte's painting (a pipe that you can
#' smoke) is feminine in the French language, a computer pipe
#' (which is an Anglicism in French) is masculine.
#'
#' @name faq-pipe-gender
NULL

#' Eager pipe
#'
#' @description
#' Whereas `%>%` is lazy and only evaluates the piped expressions when
#' needed, `%!>%` is eager and evaluates the piped input at each step.
#' This produces more intuitive behaviour when functions are called
#' for their side effects, such as displaying a message.
#'
#' Note that you can also solve this by making your function
#' strict. Call `force()` on the first argument in your function to
#' force sequential evaluation, even with the lazy `%>%` pipe.
#' See the examples section.
#'
#' @inheritParams %>%
#'
#' @examples
#' f <- function(x) {
#'   message("foo")
#'   x
#' }
#' g <- function(x) {
#'   message("bar")
#'   x
#' }
#' h <- function(x) {
#'   message("baz")
#'   invisible(x)
#' }
#'
#' # The following lazy pipe sequence is equivalent to `h(g(f()))`.
#' # Given R's lazy evaluation behaviour,`f()` and `g()` are lazily
#' # evaluated when `h()` is already running. This causes the messages
#' # to appear in reverse order:
#' NULL %>% f() %>% g() %>% h()
#'
#' # Use the eager pipe to fix this:
#' NULL %!>% f() %!>% g() %!>% h()
#'
#' # Or fix this by calling `force()` on the function arguments
#' f <- function(x) {
#'   force(x)
#'   message("foo")
#'   x
#' }
#' g <- function(x) {
#'   force(x)
#'   message("bar")
#'   x
#' }
#' h <- function(x) {
#'   force(x)
#'   message("baz")
#'   invisible(x)
#' }
#'
#' # With strict functions, the arguments are evaluated sequentially
#' NULL %>% f() %>% g() %>% h()
#'
#' # Instead of forcing, you can also check the type of your functions.
#' # Type-checking also has the effect of making your function lazy.
#'
#' @name pipe-eager
#' @export
`%!>%` <- pipe_eager_lexical

#' Debugging function for magrittr pipelines.
#'
#' This function is a wrapper around `browser`, which makes it
#' easier to debug at certain places in a magrittr pipe chain.
#'
#' @param x a value
#' @return x
#'
#' @export
debug_pipe <- function(x)
{
  browser()
  x
}

#' Debugging function for functional sequences.
#'
#' This is a utility function for marking functions in a functional 
#' sequence for debbuging.
#'
#' @param fseq a functional sequence.
#' @param ... indices of functions to debug. 
#' @return `invisible(NULL)`.
#'
#' @export
debug_fseq <- function(fseq, ...)
{
  is_valid_index <- function(i) i %in% seq_along(functions(fseq))

  indices <- list(...)
    if (!any(vapply(indices, is.numeric, logical(1L))) ||
        !any(vapply(indices, is_valid_index, logical(1L))))
      stop("Index or indices invalid.", call. = FALSE)

  invisible(lapply(indices, function(i) debug(functions(fseq)[[i]])))
}

#' @rdname debug_fseq
#' @export
undebug_fseq <- function(fseq)
{
  for (i in seq_along(functions(fseq)))
    if (isdebugged(functions(fseq)[[i]])) 
      undebug(functions(fseq)[[i]])
}

#' Aliases
#'
#' magrittr provides a series of aliases which can be more pleasant to use
#' when composing chains using the \code{\%>\%} operator.
#'
#' Currently implemented aliases are
#' \tabular{ll}{
#' \code{extract}            \tab \code{`[`}      \cr
#' \code{extract2}           \tab \code{`[[`}     \cr
#' \code{inset}              \tab \code{`[<-`}    \cr
#' \code{inset2}             \tab \code{`[[<-`}   \cr
#' \code{use_series}         \tab \code{`$`}      \cr
#' \code{add}                \tab \code{`+`}      \cr
#' \code{subtract}           \tab \code{`-`}      \cr
#' \code{multiply_by}        \tab \code{`*`}      \cr
#' \code{raise_to_power}     \tab \code{`^`}      \cr
#' \code{multiply_by_matrix} \tab \code{`\%*\%`}  \cr
#' \code{divide_by}          \tab \code{`/`}      \cr
#' \code{divide_by_int}      \tab \code{`\%/\%`}  \cr
#' \code{mod}                \tab \code{`\%\%`}   \cr
#' \code{is_in}              \tab \code{`\%in\%`} \cr
#' \code{and}                \tab \code{`&`}      \cr
#' \code{or}                 \tab \code{`|`}      \cr
#' \code{equals}             \tab \code{`==`}     \cr
#' \code{is_greater_than}    \tab \code{`>`}      \cr
#' \code{is_weakly_greater_than} \tab \code{`>=`} \cr
#' \code{is_less_than}       \tab \code{`<`}      \cr
#' \code{is_weakly_less_than}    \tab \code{`<=`} \cr
#' \code{not} (\code{`n'est pas`})  \tab \code{`!`} \cr
#' \code{set_colnames}       \tab \code{`colnames<-`} \cr
#' \code{set_rownames}       \tab \code{`rownames<-`} \cr
#' \code{set_names}          \tab \code{`names<-`} \cr
#' \code{set_class}          \tab \code{`class<-`} \cr
#' \code{set_attributes}     \tab \code{`attributes<-`} \cr
#' \code{set_attr }          \tab \code{`attr<-`} \cr
#' }
#'
#' @usage NULL
#' @export
#' @rdname aliases
#' @name extract
#' @examples
#'  iris %>%
#'    extract(, 1:4) %>%
#'    head
#'
#' good.times <-
#'   Sys.Date() %>%
#'   as.POSIXct %>%
#'   seq(by = "15 mins", length.out = 100) %>%
#'   data.frame(timestamp = .)
#'
#' good.times$quarter <-
#'   good.times %>%
#'   use_series(timestamp) %>%
#'   format("%M") %>%
#'   as.numeric %>%
#'   divide_by_int(15) %>%
#'   add(1)
extract <- `[`

#' @rdname aliases
#' @usage NULL
#' @export
extract2 <- `[[`

#' @rdname aliases
#' @usage NULL
#' @export
use_series <- `$`

#' @rdname aliases
#' @usage NULL
#' @export
add <- `+`

#' @rdname aliases
#' @usage NULL
#' @export
subtract <- `-`

#' @rdname aliases
#' @usage NULL
#' @export
multiply_by <- `*`

#' @rdname aliases
#' @usage NULL
#' @export
multiply_by_matrix <- `%*%`


#' @rdname aliases
#' @usage NULL
#' @export
divide_by <- `/`

#' @rdname aliases
#' @usage NULL
#' @export
divide_by_int <- `%/%`

#' @rdname aliases
#' @usage NULL
#' @export
raise_to_power <- `^`

#' @rdname aliases
#' @usage NULL
#' @export
and <- `&`

#' @rdname aliases
#' @usage NULL
#' @export
or <- `|`

#' @rdname aliases
#' @usage NULL
#' @export
mod <- `%%`

#' @rdname aliases
#' @usage NULL
#' @export
is_in <- `%in%`

#' @rdname aliases
#' @usage NULL
#' @export
equals <- `==`

#' @rdname aliases
#' @usage NULL
#' @export
is_greater_than <- `>`

#' @rdname aliases
#' @usage NULL
#' @export
is_weakly_greater_than <- `>=`

#' @rdname aliases
#' @usage NULL
#' @export
is_less_than <- `<`

#' @rdname aliases
#' @usage NULL
#' @export
is_weakly_less_than <- `<=`

#' @rdname aliases
#' @usage NULL
#' @export
not <- `!`

#' @rdname aliases
#' @usage NULL
#' @export
`n'est pas` <- `!`

#' @rdname aliases
#' @usage NULL
#' @export
set_colnames <- `colnames<-`

#' @rdname aliases
#' @usage NULL
#' @export
set_rownames <- `rownames<-`

#' @rdname aliases
#' @usage NULL
#' @export
set_names <- `names<-`

#' @rdname aliases
#' @usage NULL
#' @export
set_class <- `class<-`

#' @rdname aliases
#' @usage NULL
#' @export
inset <- `[<-`

#' @rdname aliases
#' @usage NULL
#' @export
inset2 <- `[[<-`

#' @rdname aliases
#' @usage NULL
#' @export
set_attr <- `attr<-`

#' @rdname aliases
#' @usage NULL
#' @export
set_attributes <- `attributes<-`

#' Extract function(s) from a functional sequence.
#'
#' Functional sequences can be subset using single or double brackets.
#' A single-bracket subset results in a new functional sequence, and 
#' a double-bracket subset results in a single function. 
#'
#' @rdname fseq
#' @param x A functional sequence
#' @param ... index/indices. For double brackets, the index must be of length 1.
#' @return A function or functional sequence.
#' @export
`[[.fseq` <- function(x, ...)
{
  functions(x)[[...]]
}

#' @rdname fseq
#' @export
`[.fseq` <- function(x, ...)
{
  y  <- x
  environment(y) <- new.env(parent = parent.env(environment(x)))
  environment(y)[["_function_list"]] <- functions(x)[...]
  y
}

context("tee pipe")

test_that("Tee pipe related functionality works.", {

  dim_message <- function(data.)
    message(sprintf("Data has dimension %d x %d", NROW(data.), NCOL(data.)))

  expect_that(iris %T>% dim_message, shows_message(dim_message(iris)))
  expect_that(iris %T>% dim_message, is_identical_to(iris))

})

context("%>%: anonymous functions on right-hand side")

test_that("%>% handles anonymous functions in GlobalEnv", {

  # Simple vectorized function
  a <- (function(x) 1 + x^2/2 + x^3/9 + x^4/16)(1:100)

  b <-
    1:100 %>%
    (function(x) 1 + x^2/2 + x^3/9 + x^4/16)

  # in principle, the dot should also work:
  c <-
    1:100 %>%
    (function(x) 1 + x^2/2 + x^3/9 + x^4/16)(.)

  expect_that(a, is_identical_to(b))
  expect_that(a, is_identical_to(c))

  # Same using preferred magrittr syntax
  a <- (function(x) 1 + x^2/2 + x^3/9 + x^4/16)(1:100)
  
  b <-
    1:100 %>%
    {1 + .^2/2 + .^3/9 + .^4/16}
    
  expect_that(a, is_identical_to(b))
  
  
  # Simple data.frame functions
  ht1 <-
    iris %>%
    (function(x) rbind(head(x), tail(x)))

  ht2 <- rbind(head(iris), tail(iris))

  expect_that(ht1, is_identical_to(ht2))


  df1 <- iris[iris$Species == "setosa", 1:4]

  df2 <-
    iris %>%
    (function(x) x[x$Species == "setosa", 1:4])

  expect_that(df1, is_identical_to(df2))


})

test_that("%>% handles anonymous functions in other situations.", {

  # Anonymous functions when %>% used in arguments.
  df1 <-
    transform(iris, test = (function(x) x^2)(Sepal.Length))

  df2 <-
    iris %>%
    transform(test = Sepal.Length %>% (function(x) x^2))

  expect_that(df1, is_identical_to(df2))


  a <- sin(abs(1:10))
  b <- sin(1:10 %>% (function(x) abs(x)))

  expect_that(a, is_identical_to(b))

  # Nested anonymous functions.
  a <- iris %>% (function(x) x[, 1] %>% (function(y) max(y)))
  b <- max(iris[, 1])

  expect_that(a, is_identical_to(b))
})


test_that("%>% throws error with anonymous functions when not parenthesized.", {
	
	expect_that(iris %>% function(x) { head(x) }, throws_error())
	
})
	

context("%>%: multi-argument functions on right-hand side")

test_that("placement of lhs is correct in different situations", {
  
  # When not to be placed in first position and in the presence of
  # non-placeholder dots, e.g. in formulas.
  case0a <- 
    lm(Sepal.Length ~ ., data = iris) %>% coef
  
  case1a <- 
    iris %>% lm(Sepal.Length ~ ., .) %>% coef
  
  case2a <-
    iris %>% lm(Sepal.Length ~ ., data = .) %>% coef
  
  expect_that(case1a, is_equivalent_to(case0a))
  expect_that(case2a, is_equivalent_to(case0a))
    
  # In first position and used in arguments
  case0b <-
    transform(iris, Species = substring(Species, 1, 1))
  
  case1b <-
    iris %>% transform(Species = Species %>% substr(1, 1))
  
  case2b <-
    iris %>% transform(., Species = Species %>% substr(., 1, 1))
  
  expect_that(case1b, is_equivalent_to(case0b))
  expect_that(case2b, is_equivalent_to(case0b))
  
  # LHS function values
  case0c <-
    aggregate(. ~ Species, iris, function(x) mean(x >= 5))
  
  case1c <-
    (function(x) mean(x >= 5)) %>% 
    aggregate(. ~ Species, iris, .)
  
  expect_that(case1c, is_equivalent_to(case0c))
  
  # several placeholder dots
  expect_true(iris %>% identical(., .))
  
  
  # "indirect" function expressions 
  expect_that(1:100 %>% iris[., ], is_identical_to(iris[1:100, ]))
  
})

test_that("exposition operator wraps `with()`", {
  out <- mtcars %>% identity() %$% (head(cyl) / mean(am))
  expect_identical(out, head(mtcars$cyl) / mean(mtcars$am))
})

test_that("compound operator works with fancy pipes", {
  data <- mtcars
  data %<>% identity %$% (head(cyl) / mean(am))
  data
  expect_identical(data, head(mtcars$cyl) / mean(mtcars$am))
})

test_that("eager pipe expressions are evaluated in the current environment", {
  rlang::local_bindings(`%>%` = pipe_eager_lexical)

  fn <- function(...) parent.frame()
  out <- NULL %>% identity() %>% fn()
  expect_identical(out, environment())

  fn <- function() {
    NULL %>% identity() %>% { return(TRUE) }
    FALSE
  }
  expect_true(fn())
})

test_that("`.` is restored", {
  1 %>% identity()
  expect_error(., "not found")

  . <- "foo"
  1 %>% identity()
  expect_identical(., "foo")
})

test_that("lazy pipe evaluates expressions lazily (#120)", {
   out <- stop("foo") %>% identity() %>% tryCatch(error = identity)
   expect_true(inherits(out, "simpleError"))

   ignore <- function(...) NA
   out <- stop("foo") %>% identity() %>% ignore()
   expect_identical(out, NA)

   out <- stop("foo") %T>% identity() %>% ignore()
   expect_identical(out, NA)

   out %<>% stop() %>% ignore()
   expect_identical(out, NA)
})

test_that("lazy pipe evaluates `.` in correct environments", {
  out <- NA %>% list(.) %>% list(.) %>% list(.)
  expect_identical(out, list(list(list(NA))))
})

test_that("nested pipe can't use multiple placeholders", {
  rlang::local_bindings(`%>%` = pipe_nested)
  expect_error(
    1 %>% list(., .),
    "multiple"
  )
})

test_that("can splice magrittr input (#191)", {
  out <- 1:3 %>% rlang::list2(!!!.)
  exp <- rlang::list2(!!!1:3)
  expect_identical(out, exp)
})

test_that("allow trailing return for backward compatibility", {
  expect_error(1 %>% { return(.) })
  expect_identical(1 %>% return(), 1)

  f <- function() 1 %>% identity() %>% return()
  expect_identical(f(), 1)
})

test_that("visibility is forwarded", {
  expect_equal(
    withVisible(mtcars %>% { identity(.$cyl) }),
    list(value = mtcars$cyl, visible = TRUE)
  )
  expect_equal(
    withVisible(mtcars %$% cyl),
    list(value = mtcars$cyl, visible = TRUE)
  )
  expect_equal(
    withVisible(mtcars %T>% identity() %>% { identity(.$cyl) }),
    list(value = mtcars$cyl, visible = TRUE)
  )

  expect_equal(
    withVisible(mtcars %>% { invisible(.$cyl) }),
    list(value = mtcars$cyl, visible = FALSE)
  )
  expect_equal(
    withVisible(mtcars %$% invisible(cyl)),
    list(value = mtcars$cyl, visible = FALSE)
  )
  expect_equal(
    withVisible(mtcars %T>% identity() %>% { invisible(.$cyl) }),
    list(value = mtcars$cyl, visible = FALSE)
  )
})

test_that("`%<>%` always returns invisibly", {
  foo <- 1
  expect_equal(
    withVisible(foo %<>% add(1) %>% identity()),
    list(value = 2, visible = FALSE)
  )
  expect_equal(
    withVisible(foo %<>% add(1) %>% invisible()),
    list(value = 3, visible = FALSE)
  )
})

test_that("internal parameters are not looked up beyond private envs of the pipes (#233)", {
  nested <- "not looked up"

  # Fails because of the duplicate placeholder if nested transformation is performed
  expect_equal(1 %>% list(., .), list(1, 1))
})

test_that("eager pipe evaluates sequentially", {
  local_edition(3)

  f <- function(x) {
    message("foo")
    x
  }
  g <- function(x) {
    message("bar")
    x
  }
  h <- function(x) {
    message("baz")
    invisible(x)
  }
  expect_snapshot({
    NULL %>% f() %>% g() %>% h()
    NULL %!>% f() %!>% g() %!>% h()
  })
})

context("assignment pipe")

test_that("Assignment pipe works", {

  x <- y <- 1:10
  x[1:5] <- sin(cos(x[1:5]))
  y[1:5] %<>% cos %>% sin

  expect_that(x, is_identical_to(y))


  somedata <- iris
  somedata$Sepal.Length %<>% add(10)
  iris$Sepal.Length <- iris$Sepal.Length + 10

  expect_that(somedata, is_identical_to(iris))

  z <- 1:10
  z %<>% add(2) %T>% plot
  expect_that(z, is_identical_to(as.numeric(3:12)))

})

test_that("can't assign to non-assignment expression", {
  msg <- conditionMessage(tryCatch(error = identity, identity(1) <- NULL))
  expect_error(
    1 %>% identity() %<>% identity(),
    msg
  )
})

context("magrittr aliases")

test_that("the provided aliases work as intended.", {

  expect_that(iris %>% extract(, 1:2), is_identical_to(iris[, 1:2]))
  expect_that(iris %>% extract2(1),    is_identical_to(iris[[1]]))
  expect_that(iris %>% use_series(Species), is_identical_to(iris$Species))
  expect_that(1:10 %>% add(10:1), is_identical_to(1:10 + 10:1))
  expect_that(1:10 %>% subtract(10:1), is_identical_to(1:10 - 10:1))
  expect_that(1:10 %>% multiply_by(10:1), is_identical_to(1:10 * 10:1))

  A <- matrix(1:16, 4, 4)
  expect_that(A %>% multiply_by_matrix(A), is_identical_to(A %*% A))
  expect_that(1:10 %>% raise_to_power(10:1), is_identical_to((1:10)^(10:1)))
  expect_that(1:10 %>% divide_by(10:1), is_identical_to(1:10 / 10:1))
  expect_that(1:10 %>% divide_by_int(10:1), is_identical_to(1:10 %/% 10:1))
  expect_that(1:10 %>% mod(3), is_identical_to((1:10) %% 3))
  expect_that(((1:10) > 5) %>% and((1:10) > 7),
              is_identical_to(((1:10) > 5) & (1:10) > 7))
  expect_that(((1:10) > 5) %>% or((1:10) > 7),
              is_identical_to(((1:10) > 5) | (1:10) > 7))

  expect_that(1:10 %>% (magrittr::equals)(5) %>% sum, is_identical_to(1L))
  expect_that(1:10 %>% is_greater_than(5) %>% sum, is_identical_to(5L))
  expect_that(1:10 %>% is_weakly_greater_than(5) %>% sum, is_identical_to(6L))
  expect_that(1:10 %>% (magrittr::is_less_than)(5) %>% sum, is_identical_to(4L))
  expect_that(1:10 %>% is_weakly_less_than(5) %>% sum, is_identical_to(5L))

  expect_that(iris %>% set_colnames(LETTERS[1:ncol(iris)]),
              is_identical_to(`colnames<-`(iris, LETTERS[1:ncol(iris)])))

  expect_that(1:10 %>% set_names(LETTERS[1:10]),
              is_identical_to(`names<-`(1:10, LETTERS[1:10])))

  expect_that(diag(3) %>% set_rownames(c("x", "y", "z")),
              is_identical_to(`rownames<-`(diag(3), c("x", "y", "z"))))

  expect_that(1:10 %>% is_greater_than(5) %>% not,
              is_identical_to(1:10 %>% is_weakly_less_than(5)))

})

context("%>%: one-argument function alternatives.")

test_that("%>% works as expected with and without parentheses and placeholder", {
  
  expect_that(1:100 %>% sin %>% abs,       is_identical_to(abs(sin(1:100))))
  expect_that(1:100 %>% sin() %>% abs(),   is_identical_to(abs(sin(1:100))))
  expect_that(1:100 %>% sin(.) %>% abs(.), is_identical_to(abs(sin(1:100))))
  
  expect_that(iris %>% head, is_identical_to(head(iris)))
  
  dnormsd <- function(sd) function(x) dnorm(x, sd = sd)
  some_x  <- rnorm(20)
  expect_that(some_x %>% dnormsd(5)(.), is_identical_to(dnormsd(5)(some_x)))
  expect_that(some_x %>% (dnormsd(5)), is_identical_to(dnormsd(5)(some_x)))
  
  expect_that(some_x %>% dnormsd(5), throws_error())
  expect_that(some_x %>% function(x) {x} %>% sin, throws_error())
})

---
title: "Introducing magrittr"
author: Stefan Milton Bache
date: November, 2014
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introducing magrittr}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

```{r include = FALSE}
library(magrittr)
options(scipen = 3)
knitr::opts_chunk$set(comment = "#>", collapse = TRUE)
```

# Abstract

The *magrittr* (to be pronounced with a sophisticated french accent) package has
two aims: decrease development time and improve readability and maintainability 
of code. Or even shortr: make your code smokin' (puff puff)!

To achieve its humble aims, *magrittr* (remember the accent) provides a new 
"pipe"-like operator, `%>%`, with which you may pipe a value forward into an 
expression or function call; something along the lines of ` x %>% f `, rather
than ` f(x)`. This is not an unknown feature elsewhere; a prime example is the 
`|>` operator used extensively in `F#` (to say the least) and indeed this 
-- along with Unix pipes -- served as a motivation for developing the magrittr 
package.

This vignette describes the main features of *magrittr* and demonstrates
some features which have been added since the initial release.

# Introduction and basics

At first encounter, you may wonder whether an operator such as `%>%` can really 
be all that beneficial; but as you may notice, it semantically changes your 
code in a way that makes it more intuitive to both read and write.

Consider the following example, in which the `mtcars` dataset shipped with
R is munged a little:
```{r}
library(magrittr)

car_data <- 
  mtcars %>%
  subset(hp > 100) %>%
  aggregate(. ~ cyl, data = ., FUN = . %>% mean %>% round(2)) %>%
  transform(kpl = mpg %>% multiply_by(0.4251)) %>%
  print
```
We start with a value, here `mtcars` (a `data.frame`). From there, we extract a 
subset, aggregate the information based on the number of cylinders, and then 
transform the dataset by adding a variable for kilometers per liter as a 
supplement to miles per gallon. Finally we print the result before assigning 
it. Note how the code is arranged in the logical order of how you think about 
the task: data->transform->aggregate, which is also the same order as the code 
will execute. It's like a recipe -- easy to read, easy to follow!

A horrific alternative would be to write:
```{r}
car_data <- 
  transform(aggregate(. ~ cyl, 
                      data = subset(mtcars, hp > 100), 
                      FUN = function(x) round(mean(x), 2)), 
            kpl = mpg*0.4251)
```
There is a lot more clutter with parentheses, and the mental task of deciphering
the code is more challenging---particularly if you did not write it yourself.

Note also how "building" a function on the fly for use in `aggregate` is very
simple in *magrittr*: rather than an actual value as the left-hand side in 
the pipeline, just use the placeholder. This is also very useful in R's 
`*apply` family of functions.

Granted, you may make the second example better, perhaps throw in a few 
temporary variables (which is often avoided to some degree when using 
*magrittr*), but one often sees cluttered lines like the ones presented. 

And here is another selling point: suppose I want to quickly add another step 
somewhere in the process. This is very easy to do in the pipeline version, but 
a little more challenging in the "standard" example.

The combined example shows a few neat features of the pipe (which it is not):

1. By default the left-hand side (LHS) will be *piped in* as the first argument of 
the function appearing on the right-hand side (RHS). This is the case in the 
`subset` and `transform` expressions.
2. `%>%` may be used in a nested fashion, e.g. it may appear in expressions within 
arguments. This is illustrated in the `mpg` to `kpl` conversion.
3. When the LHS is needed at a position other than the first, one can use
the dot,`'.'`, as placeholder. This is shown in the `aggregate` expression.
4. The dot in e.g. a formula is *not* confused with a placeholder, which is
utilized in the `aggregate` expression.
5. Whenever only *one* argument (the LHS) is needed, one can omit the empty 
parentheses. This is shown in the call to `print` (which also returns its
argument). Here, `LHS %>% print()`, or even `LHS %>% print(.)` would also work.
6. A pipeline with a dot (`.`) as the LHS will create a unary function. This is
used to define the aggregator function.

One feature, which was not demonstrated above is piping into *anonymous 
functions*, or *lambdas*. This is possible using standard function definitions, 
e.g.:
```{r, eval = FALSE}
car_data %>%
(function(x) {
  if (nrow(x) > 2) 
    rbind(head(x, 1), tail(x, 1))
  else x
})
```
However, *magrittr* also allows a short-hand notation:
```{r}
car_data %>%
{ 
  if (nrow(.) > 0)
    rbind(head(., 1), tail(., 1))
  else .
}
```
Since all right-hand sides are really "body expressions" of unary functions, 
this is only the natural extension of the simple right-hand side expressions. 
Of course, longer and more complex functions can be made using this approach.

In the first example, the anonymous function is enclosed in parentheses.
Whenever you want to use a function- or call-generating statement as right-hand 
side, parentheses are used to evaluate the right-hand side before piping takes 
place.

Another, less useful example is:
```{r}
1:10 %>% (substitute(f(), list(f = sum)))
```

# Additional pipe operators
*magrittr* also provides three related pipe operators. These are not as common 
as `%>%` but they become useful in special cases. 

The "tee" pipe, `%T>%` works like `%>%`, except it returns the left-hand
side value, and not the result of the right-hand side operation. This is useful 
when a step in a pipeline is used for its side-effect (printing, plotting, 
logging, etc.). As an example (where the actual plot is omitted here):
```{r, fig.keep='none'}
rnorm(200) %>%
matrix(ncol = 2) %T>%
plot %>% # plot usually does not return anything. 
colSums
```

The "exposition" pipe, `%$%` exposes the names within the left-hand side
object to the right-hand side expression. Essentially, it is a short-hand for 
using the `with` functions (and the same left-hand side objects are accepted).
This operator is handy when functions do not themselves have a data argument, as 
for example `lm` and `aggregate` do. Here are a few examples as illustration:

```{r, eval = FALSE}
iris %>%
  subset(Sepal.Length > mean(Sepal.Length)) %$%
  cor(Sepal.Length, Sepal.Width)
   
data.frame(z = rnorm(100)) %$% 
  ts.plot(z)
```

Finally, the "assignment" pipe `%<>%` can be used as the first 
pipe in a chain. The effect will be that the result of the pipeline is assigned 
to the left-hand side object, rather than returning the result as usual. It is 
essentially shorthand notation for expressions like `foo <- foo %>% bar %>% baz`, 
which boils down to `foo %<>% bar %>% baz`. Another example is:

```{r, eval = FALSE}
iris$Sepal.Length %<>% sqrt
```

The `%<>%` can be used whenever `expr <- ...` makes sense, e.g. 

* `x %<>% foo %>% bar`
* `x[1:10] %<>% foo %>% bar`
* `x$baz %<>% foo %>% bar`

# Aliases

In addition to the `%>%`-operator, *magrittr* provides some aliases for other
operators which make operations such as addition or multiplication fit well 
into the *magrittr*-syntax. As an example, consider:
```{r}
rnorm(1000)    %>%
multiply_by(5) %>%
add(5)         %>%
{ 
   cat("Mean:", mean(.), 
       "Variance:", var(.), "\n")
   head(.)
}
```
which could be written in more compact form as:
```{r, results = 'hide'}
rnorm(100) %>% `*`(5) %>% `+`(5) %>% 
{
  cat("Mean:", mean(.), "Variance:", var(.),  "\n")
  head(.)
}
```
To see a list of the aliases, execute e.g. `?multiply_by`. 

# Development
The *magrittr* package is also available in a development version at the
GitHub development page:
[github.com/tidyverse/magrittr](https://github.com/tidyverse/magrittr).

# eager pipe evaluates sequentially

    Code
      NULL %>% f() %>% g() %>% h()
    Message <simpleMessage>
      baz
      bar
      foo
    Code
      NULL %!>% f() %!>% g() %!>% h()
    Message <simpleMessage>
      foo
      bar
      baz

# eager pipe evaluates sequentially

    Code
      NULL %>% f() %>% g() %>% h()
    Message
      baz
      bar
      foo
    Code
      NULL %!>% f() %!>% g() %!>% h()
    Message
      foo
      bar
      baz

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/debug_pipe.R
\name{debug_pipe}
\alias{debug_pipe}
\title{Debugging function for magrittr pipelines.}
\usage{
debug_pipe(x)
}
\arguments{
\item{x}{a value}
}
\value{
x
}
\description{
This function is a wrapper around \code{browser}, which makes it
easier to debug at certain places in a magrittr pipe chain.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe}
\usage{
lhs \%>\% rhs
}
\arguments{
\item{lhs}{A value or the magrittr placeholder.}

\item{rhs}{A function call using the magrittr semantics.}
}
\description{
Pipe an object forward into a function or call expression.
}
\details{
\subsection{Using \verb{\%>\%} with unary function calls}{

When functions require only one argument, \code{x \%>\% f} is equivalent
to \code{f(x)} (not exactly equivalent; see technical note below.)

}
\subsection{Placing \code{lhs} as the first argument in \code{rhs} call}{

The default behavior of \verb{\%>\%} when multiple arguments are required
in the \code{rhs} call, is to place \code{lhs} as the first argument, i.e.
\code{x \%>\% f(y)} is equivalent to \code{f(x, y)}.
}
\subsection{Placing \code{lhs} elsewhere in \code{rhs} call}{

Often you will want \code{lhs} to the \code{rhs} call at another position than the first.
For this purpose you can use the dot (\code{.}) as placeholder. For example,
\code{y \%>\% f(x, .)} is equivalent to \code{f(x, y)} and
\code{z \%>\% f(x, y, arg = .)} is equivalent to \code{f(x, y, arg = z)}.
}

\subsection{Using the dot for secondary purposes}{

Often, some attribute or property of \code{lhs} is desired in the \code{rhs} call in
addition to the value of \code{lhs} itself, e.g. the number of rows or columns.
It is perfectly valid to use the dot placeholder several times in the \code{rhs}
call, but by design the behavior is slightly different when using it inside
nested function calls. In particular, if the placeholder is only used
in a nested function call, \code{lhs} will also be placed as the first argument!
The reason for this is that in most use-cases this produces the most readable
code. For example, \code{iris \%>\% subset(1:nrow(.) \%\% 2 == 0)} is
equivalent to \code{iris \%>\% subset(., 1:nrow(.) \%\% 2 == 0)} but
slightly more compact. It is possible to overrule this behavior by enclosing
the \code{rhs} in braces. For example, \code{1:10 \%>\% {c(min(.), max(.))}} is
equivalent to \code{c(min(1:10), max(1:10))}.
}

\subsection{Using \verb{\%>\%} with call- or function-producing \code{rhs}}{

It is possible to force evaluation of \code{rhs} before the piping of \code{lhs} takes
place. This is useful when \code{rhs} produces the relevant call or function.
To evaluate \code{rhs} first, enclose it in parentheses, i.e.
\code{a \%>\% (function(x) x^2)}, and \code{1:10 \%>\% (call("sum"))}.
Another example where this is relevant is for reference class methods
which are accessed using the \code{$} operator, where one would do
\code{x \%>\% (rc$f)}, and not \code{x \%>\% rc$f}.
}

\subsection{Using lambda expressions with \verb{\%>\%}}{

Each \code{rhs} is essentially a one-expression body of a unary function.
Therefore defining lambdas in magrittr is very natural, and as
the definitions of regular functions: if more than a single expression
is needed one encloses the body in a pair of braces, \code{\{ rhs \}}.
However, note that within braces there are no "first-argument rule":
it will be exactly like writing a unary function where the argument name is
"\code{.}" (the dot).

}
\subsection{Using the dot-place holder as \code{lhs}}{

When the dot is used as \code{lhs}, the result will be a functional sequence,
i.e. a function which applies the entire chain of right-hand sides in turn
to its input. See the examples.
}
}
\section{Technical notes}{

The magrittr pipe operators use non-standard evaluation. They capture
their inputs and examines them to figure out how to proceed. First a function
is produced from all of the individual right-hand side expressions, and
then the result is obtained by applying this function to the left-hand side.
For most purposes, one can disregard the subtle aspects of magrittr's
evaluation, but some functions may capture their calling environment,
and thus using the operators will not be exactly equivalent to the
"standard call" without pipe-operators.

Another note is that special attention is advised when using non-magrittr
operators in a pipe-chain (\verb{+, -, $,} etc.), as operator precedence will impact how the
chain is evaluated. In general it is advised to use the aliases provided
by magrittr.
}

\examples{
# Basic use:
iris \%>\% head

# Use with lhs as first argument
iris \%>\% head(10)

# Using the dot place-holder
"Ceci n'est pas une pipe" \%>\% gsub("une", "un", .)
  
# When dot is nested, lhs is still placed first:
sample(1:10) \%>\% paste0(LETTERS[.])

# This can be avoided:
rnorm(100) \%>\% {c(min(.), mean(.), max(.))} \%>\% floor

# Lambda expressions: 
iris \%>\%
{
  size <- sample(1:10, size = 1)
  rbind(head(., size), tail(., size))
}

# renaming in lambdas:
iris \%>\%
{
  my_data <- .
  size <- sample(1:10, size = 1)
  rbind(head(my_data, size), tail(my_data, size))
}

# Building unary functions with \%>\%
trig_fest <- . \%>\% tan \%>\% cos \%>\% sin

1:10 \%>\% trig_fest
trig_fest(1:10)

}
\seealso{
\code{\link{\%<>\%}}, \code{\link{\%T>\%}}, \code{\link{\%$\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{extract}
\alias{extract}
\alias{extract2}
\alias{use_series}
\alias{add}
\alias{subtract}
\alias{multiply_by}
\alias{multiply_by_matrix}
\alias{divide_by}
\alias{divide_by_int}
\alias{raise_to_power}
\alias{and}
\alias{or}
\alias{mod}
\alias{is_in}
\alias{equals}
\alias{is_greater_than}
\alias{is_weakly_greater_than}
\alias{is_less_than}
\alias{is_weakly_less_than}
\alias{not}
\alias{n'est pas}
\alias{set_colnames}
\alias{set_rownames}
\alias{set_names}
\alias{set_class}
\alias{inset}
\alias{inset2}
\alias{set_attr}
\alias{set_attributes}
\title{Aliases}
\description{
magrittr provides a series of aliases which can be more pleasant to use
when composing chains using the \code{\%>\%} operator.
}
\details{
Currently implemented aliases are
\tabular{ll}{
\code{extract}            \tab \code{`[`}      \cr
\code{extract2}           \tab \code{`[[`}     \cr
\code{inset}              \tab \code{`[<-`}    \cr
\code{inset2}             \tab \code{`[[<-`}   \cr
\code{use_series}         \tab \code{`$`}      \cr
\code{add}                \tab \code{`+`}      \cr
\code{subtract}           \tab \code{`-`}      \cr
\code{multiply_by}        \tab \code{`*`}      \cr
\code{raise_to_power}     \tab \code{`^`}      \cr
\code{multiply_by_matrix} \tab \code{`\%*\%`}  \cr
\code{divide_by}          \tab \code{`/`}      \cr
\code{divide_by_int}      \tab \code{`\%/\%`}  \cr
\code{mod}                \tab \code{`\%\%`}   \cr
\code{is_in}              \tab \code{`\%in\%`} \cr
\code{and}                \tab \code{`&`}      \cr
\code{or}                 \tab \code{`|`}      \cr
\code{equals}             \tab \code{`==`}     \cr
\code{is_greater_than}    \tab \code{`>`}      \cr
\code{is_weakly_greater_than} \tab \code{`>=`} \cr
\code{is_less_than}       \tab \code{`<`}      \cr
\code{is_weakly_less_than}    \tab \code{`<=`} \cr
\code{not} (\code{`n'est pas`})  \tab \code{`!`} \cr
\code{set_colnames}       \tab \code{`colnames<-`} \cr
\code{set_rownames}       \tab \code{`rownames<-`} \cr
\code{set_names}          \tab \code{`names<-`} \cr
\code{set_class}          \tab \code{`class<-`} \cr
\code{set_attributes}     \tab \code{`attributes<-`} \cr
\code{set_attr }          \tab \code{`attr<-`} \cr
}
}
\examples{
 iris \%>\%
   extract(, 1:4) \%>\%
   head

good.times <-
  Sys.Date() \%>\%
  as.POSIXct \%>\%
  seq(by = "15 mins", length.out = 100) \%>\%
  data.frame(timestamp = .)

good.times$quarter <-
  good.times \%>\%
  use_series(timestamp) \%>\%
  format("\%M") \%>\%
  as.numeric \%>\%
  divide_by_int(15) \%>\%
  add(1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{faq-pipe-gender}
\alias{faq-pipe-gender}
\title{FAQ: What is the gender of the pipe?}
\description{
In Magritte's original quote "Ceci n'est pas une pipe," the word
"pipe" is feminine. However the magrittr package quotes it as
"Ceci n'est pas un pipe," with a masculine "pipe." This
lighthearted misappropriation is intentional. Whereas the
object represented in Magritte's painting (a pipe that you can
smoke) is feminine in the French language, a computer pipe
(which is an Anglicism in French) is masculine.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/debug_pipe.R
\name{debug_fseq}
\alias{debug_fseq}
\alias{undebug_fseq}
\title{Debugging function for functional sequences.}
\usage{
debug_fseq(fseq, ...)

undebug_fseq(fseq)
}
\arguments{
\item{fseq}{a functional sequence.}

\item{...}{indices of functions to debug.}
}
\value{
\code{invisible(NULL)}.
}
\description{
This is a utility function for marking functions in a functional
sequence for debbuging.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%T>\%}
\alias{\%T>\%}
\title{Tee pipe}
\usage{
lhs \%T>\% rhs
}
\arguments{
\item{lhs}{A value or the magrittr placeholder.}

\item{rhs}{A function call using the magrittr semantics.}
}
\description{
Pipe a value forward into a function- or call expression and return the
original value instead of the result. This is useful when an expression
is used for its side-effect, say plotting or printing.
}
\details{
The tee pipe works like \code{\link{\%>\%}}, except the
return value is \code{lhs} itself, and not the result of \code{rhs} function/expression.
}
\examples{
rnorm(200) \%>\%
matrix(ncol = 2) \%T>\%
plot \%>\% # plot usually does not return anything. 
colSums

}
\seealso{
\code{\link{\%>\%}}, \code{\link{\%<>\%}}, \code{\link{\%$\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{pipe_eager_lexical}
\alias{pipe_eager_lexical}
\alias{pipe_lazy_masking}
\alias{pipe_nested}
\title{Lazy and eager pipes}
\usage{
pipe_eager_lexical(lhs, rhs)

pipe_lazy_masking(lhs, rhs)

pipe_nested(lhs, rhs)
}
\arguments{
\item{lhs}{A value or the magrittr placeholder.}

\item{rhs}{A function call using the magrittr semantics.}
}
\description{
Assign these pipe variants to an infix symbol like \verb{\%>\%}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magrittr.R
\docType{package}
\name{magrittr-package}
\alias{magrittr}
\alias{magrittr-package}
\title{magrittr - Ceci n'est pas un pipe}
\description{
The magrittr package offers a set of operators which promote semantics
that will improve your code by
\itemize{
\item structuring sequences of data operations left-to-right
(as opposed to from the inside and out),
\item avoiding nested function calls,
\item minimizing the need for local variables and function definitions, and
\item making it easy to add steps anywhere in the sequence of operations.
}
The operators pipe their left-hand side values forward into expressions that
appear on the right-hand side, i.e. one can replace \code{f(x)} with
\code{x \%>\% f}, where \code{\%>\%} is the (main) pipe-operator.
}
\details{
Consider the example below. Four operations are performed to
arrive at the desired data set, and they are written in a natural order:
the same as the order of execution. Also, no temporary variables are needed.
If yet another operation is required, it is straight-forward to add to the
sequence of operations whereever it may be needed.

For a more detailed introduction see the vignette
(\code{vignette("magrittr")}) or the documentation pages for the
available operators:\cr
\tabular{ll}{
\code{\link{\%>\%}}  \tab pipe.\cr
\code{\link{\%T>\%}} \tab tee pipe.\cr
\code{\link{\%<>\%}} \tab assignment pipe.\cr
\code{\link{\%$\%}}  \tab exposition pipe.\cr
}
}
\examples{
\dontrun{

the_data <-
  read.csv('/path/to/data/file.csv') \%>\%
  subset(variable_a > x) \%>\%
  transform(variable_c = variable_a/variable_b) \%>\%
  head(100)
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://magrittr.tidyverse.org}
  \item \url{https://github.com/tidyverse/magrittr}
  \item Report bugs at \url{https://github.com/tidyverse/magrittr/issues}
}

}
\author{
\strong{Maintainer}: Lionel Henry \email{lionel@posit.co}

Authors:
\itemize{
  \item Stefan Milton Bache \email{stefan@stefanbache.dk} (Original author and creator of magrittr) [copyright holder]
  \item Hadley Wickham \email{hadley@posit.co}
}

Other contributors:
\itemize{
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getters.R
\name{[[.fseq}
\alias{[[.fseq}
\alias{[.fseq}
\title{Extract function(s) from a functional sequence.}
\usage{
\method{[[}{fseq}(x, ...)

\method{[}{fseq}(x, ...)
}
\arguments{
\item{x}{A functional sequence}

\item{...}{index/indices. For double brackets, the index must be of length 1.}
}
\value{
A function or functional sequence.
}
\description{
Functional sequences can be subset using single or double brackets.
A single-bracket subset results in a new functional sequence, and
a double-bracket subset results in a single function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{print.fseq}
\alias{print.fseq}
\title{Print method for functional sequence.}
\usage{
\method{print}{fseq}(x, ...)
}
\arguments{
\item{x}{A functional sequence object}

\item{...}{not used.}
}
\value{
x
}
\description{
Print method for functional sequence.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%$\%}
\alias{\%$\%}
\title{Exposition pipe}
\usage{
lhs \%$\% rhs
}
\arguments{
\item{lhs}{A list, environment, or a data.frame.}

\item{rhs}{An expression where the names in lhs is available.}
}
\description{
Expose the names in \code{lhs} to the \code{rhs} expression. This is useful when functions
do not have a built-in data argument.
}
\details{
Some functions, e.g. \code{lm} and \code{aggregate}, have a
data argument, which allows the direct use of names inside the data as part
of the call. This operator exposes the contents of the left-hand side object
to the expression on the right to give a similar benefit, see the examples.
}
\examples{
iris \%>\%
  subset(Sepal.Length > mean(Sepal.Length)) \%$\%
  cor(Sepal.Length, Sepal.Width)
  
data.frame(z = rnorm(100)) \%$\% 
  ts.plot(z)
  
}
\seealso{
\code{\link{\%>\%}}, \code{\link{\%<>\%}}, \code{\link{\%T>\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{functions}
\alias{functions}
\title{Extract the function list from a functional sequence.}
\usage{
functions(fseq)
}
\arguments{
\item{fseq}{A functional sequence ala magrittr.}
}
\value{
a list of functions
}
\description{
This can be used to extract the list of functions inside a functional
sequence created with a chain like \code{. \%>\% foo \%>\% bar}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%<>\%}
\alias{\%<>\%}
\title{Assignment pipe}
\usage{
lhs \%<>\% rhs
}
\arguments{
\item{lhs}{An object which serves both as the initial value and as target.}

\item{rhs}{a function call using the magrittr semantics.}
}
\description{
Pipe an object forward into a function or call expression and update the
\code{lhs} object with the resulting value.
}
\details{
The assignment pipe, \code{\%<>\%}, is used to
update a value by first piping it into one or more \code{rhs} expressions, and
then assigning the result. For example, \code{some_object \%<>\% 
foo \%>\% bar} is equivalent to \code{some_object <- some_object \%>\% foo
\%>\% bar}. It must be the first pipe-operator in a chain, but otherwise it
works like \code{\link{\%>\%}}.
}
\examples{
iris$Sepal.Length \%<>\% sqrt

x <- rnorm(100)

x \%<>\% abs \%>\% sort

is_weekend <- function(day)
{
   # day could be e.g. character a valid representation
   day \%<>\% as.Date
   
   result <- day \%>\% format("\%u") \%>\% as.numeric \%>\% is_greater_than(5)
   
   if (result)
     message(day \%>\% paste("is a weekend!"))
   else
     message(day \%>\% paste("is not a weekend!"))
   
   invisible(result)
}

}
\seealso{
\code{\link{\%>\%}}, \code{\link{\%T>\%}}, \code{\link{\%$\%}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freduce.R
\name{freduce}
\alias{freduce}
\title{Apply a list of functions sequentially}
\usage{
freduce(value, function_list)
}
\arguments{
\item{value}{initial value.}

\item{function_list}{a list of functions.}
}
\value{
The result after applying each function in turn.
}
\description{
This function applies the first function to \code{value}, then the
next function to the result of the previous function call, etc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{pipe-eager}
\alias{pipe-eager}
\alias{\%!>\%}
\title{Eager pipe}
\usage{
lhs \%!>\% rhs
}
\arguments{
\item{lhs}{A value or the magrittr placeholder.}

\item{rhs}{A function call using the magrittr semantics.}
}
\description{
Whereas \verb{\%>\%} is lazy and only evaluates the piped expressions when
needed, \verb{\%!>\%} is eager and evaluates the piped input at each step.
This produces more intuitive behaviour when functions are called
for their side effects, such as displaying a message.

Note that you can also solve this by making your function
strict. Call \code{force()} on the first argument in your function to
force sequential evaluation, even with the lazy \verb{\%>\%} pipe.
See the examples section.
}
\examples{
f <- function(x) {
  message("foo")
  x
}
g <- function(x) {
  message("bar")
  x
}
h <- function(x) {
  message("baz")
  invisible(x)
}

# The following lazy pipe sequence is equivalent to `h(g(f()))`.
# Given R's lazy evaluation behaviour,`f()` and `g()` are lazily
# evaluated when `h()` is already running. This causes the messages
# to appear in reverse order:
NULL \%>\% f() \%>\% g() \%>\% h()

# Use the eager pipe to fix this:
NULL \%!>\% f() \%!>\% g() \%!>\% h()

# Or fix this by calling `force()` on the function arguments
f <- function(x) {
  force(x)
  message("foo")
  x
}
g <- function(x) {
  force(x)
  message("bar")
  x
}
h <- function(x) {
  force(x)
  message("baz")
  invisible(x)
}

# With strict functions, the arguments are evaluated sequentially
NULL \%>\% f() \%>\% g() \%>\% h()

# Instead of forcing, you can also check the type of your functions.
# Type-checking also has the effect of making your function lazy.

}
